using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Collections.Generic;
using System.Text;

namespace SDPluginRFID
{
    
    /// <summary>
    /// Class used to resolve URLs read from RFID tags, by fetching the xml-document the URL
    /// point to and decode the service information in these.
    /// </summary>
    class WebResolver
    {
        
        //Buffer for read operations
        byte[] buf = new byte[8192];



        /// <summary>
        /// Method to fetch a web-page containing a xml-document
        /// </summary>
        /// <param name="URL">The URL to resolve</param>
        /// <returns>The service-list</returns>
        public List<Service> ResolveURL(string URL)
        {
            //Create some needed variables
            StringBuilder sb = new StringBuilder();
            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(URL);
            HttpWebResponse response = (HttpWebResponse) request.GetResponse();
            Stream resStream = response.GetResponseStream();

            string tempString = null;
            int count = 0;

            do
            {
                count = resStream.Read(buf, 0, buf.Length);
                if (count != 0)
                {
                    tempString = Encoding.ASCII.GetString(buf, 0, count);
                    sb.Append(tempString);
                }
            }
            while (count > 0);

            return ParseServiceList( sb.ToString() );
        }



        /// <summary>
        /// Method to convert a xml-document (from a string) to a List with Service objects
        /// </summary>
        /// <param name="s">Must be a valid xml-formated list containing the services</param>
        /// <returns>The service-list</returns>
        private List<Service> ParseServiceList(String xmlString)
        {
            List<Service> items = new List<Service>();
            XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(xmlString));

            while (reader.Read())
            {				
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Service" && reader.AttributeCount > 0)
                {
                    Service s = new Service();
                    
                    s.Name = reader.GetAttribute("Name");
                    s.ServiceUri = reader.GetAttribute("ServiceUri");
                    s.Location = reader.GetAttribute("Location");
                    s.Owner = reader.GetAttribute("Owner");
                    s.DescriptionUri = reader.GetAttribute("DescriptionUri");
                    s.Type = reader.GetAttribute("Type");
                    s.Description = reader.GetAttribute("Description");

                    items.Add(s);
                }
            }

            return items;
        }

    }
}
