/*
 * UbiCollab Service Discovery Plugin for Service Registry is developed by 
 * Kim-Steve Johansen as part of a master thesis at The Norwegian University 
 * of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

/**
 * The Class Activator contains the start method, which is the method called
 * by the framework to start/activate this bundle (and then the registry service).
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class Activator implements BundleActivator {

	/** The Service Discovery plugin for Service Registry. */
	private PluginServiceRegistryImpl srPlugin = null;

	
	/**
	 * This method is invoked by the framework to start the bundle and its service 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			TimeKeeper.resetThread();
			srPlugin = new PluginServiceRegistryImpl();
			System.out.println("SDPlugin for Service Registry has started.");
			log.writeToLog("SDPlugin for Service Registry has started.", Log.INFO);
		} 
		catch (Exception e) {
			System.out.println("SDPlugin for Service Registry failed to start.");
			log.writeToLog("SDPlugin for Service Registry failed to start.", Log.WARNING);
			throw new BundleException("Failed to start SDPlugin for Service Registry.", e);
		}
	}
	
	
	/**
	 * This method is invoked by the framework to stop the bundle and its service.
	 * All resources used is released to the garbage collector. 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			TimeKeeper.stopThread();
			if (srPlugin != null)
				srPlugin = null;
			System.out.println("SDPlugin for Service Registry has stopped.");
			log.writeToLog("SDPlugin for Service Registry has stopped.", Log.INFO);
		} 
		catch (Exception e) {
			System.out.println("SDPlugin for Service Registry failed to stop.");
			log.writeToLog("SDPlugin for Service Registry failed to stop.", Log.WARNING);
			throw new BundleException("Failed to stop Plugin for Service Registry", e);
		}
	}

}