/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

/**
 * The Class GlobalSettings holds a set of constants that will be used by other classes
 * throughout the Service Registry Module. This gives a single point of configuration.
 * 
 * @author Kim-Steve Johansen
 * @version 0.3
 * @since 0.1
 */
public class GlobalSettings {
	
	/** The Constant SOAP_SERVICE_NAME is the service name that will be exported. */
	public final static String SOAP_SERVICE_NAME = "discoveryManager";
	
	/** The Constant STORAGE_ROOT holds the name of the ubicollab file folder. */
	public final static String STORAGE_ROOT = "ubifiles";
	
	/** The Constant LOG_FILE holds the name for the SR log file. */
	public final static String LOG_FILE = "log_servicediscovery.xml";
	
	/** The Constant SELECTED_LOG_LEVEL decides which level to log. */
	public final static int SELECTED_LOG_LEVEL = Log.DEBUG;
	
	/** The Constant PLUGIN_TIME_TO_LIVE set the maximum time that plugins 
	 * can omit to have contact with Service Discovery. */
	public final static int PLUGIN_TIME_TO_LIVE = 30000;
	
	/** The Constant CLIENT_TIME_TO_LIVE set the maximum time that clients 
	 * can omit to have contact with Service Discovery. */
	public final static int CLIENT_TIME_TO_LIVE = 60000;

	/** The valid XML Header string. */
	public final static String XML_HEADER = "<?xml version=\"1.0\"?>";

}
