/*
 * UbiCollab Service Discovery Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is freely distributable under the Apache License v2.0.
 * 
 * For more information, contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

/**
 * The Class HouseKeeperThread run as a seperate thread and is responsible for picking out
 * clients and plugins which have not been in contact for a time duration specified in
 * <code>GlobalSettings</code>.
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class HouseKeeperThread extends Thread {
	
	/** The time between every decrease of time-to-live for clients and consumers . */
	private static int TIME_BETWEEN_HOUSEKEEPING_TASKS = 5000;
	
	/** While <code>running</code> is <code>true</code> the <code>HouseKeeperThread</code> keeps running. */
	private static boolean running = true;
	
	/** The plugin list. */
	private PluginList pluginList;
	
	/** The client list. */
	private ClientList clientList;
	
	
	public HouseKeeperThread(PluginList pList, ClientList cList) {
		pluginList = pList;
		clientList = cList;
	}
	
	
	
	/**
	 * The code in this method starts to run as a thread when the inherited 
	 * <code>Thread#start</code> have been called. 
	 * 
	 * @see java.lang.Thread#run()
	 */
	public void run() {
		while (running) {
			
			pluginList.decreaseTimeToLive(TIME_BETWEEN_HOUSEKEEPING_TASKS);
			clientList.decreaseTimeToLive(TIME_BETWEEN_HOUSEKEEPING_TASKS);
			
			try {
				Thread.sleep(TIME_BETWEEN_HOUSEKEEPING_TASKS);
			} 
			catch (InterruptedException ie) {
				ie.printStackTrace();
			}
		}
	}
	

	
	/**
	 * This method will stop the <code>HouseKeeperThread</code> thread.
	 */
	public static void stopThread() {
		running = false;
	}
	
	
	
	/**
	 * This method will reset the running member of the 
	 * <code>HouseKeeperThread</code> thread. This must be
	 * done before the thread can be started again.
	 */
	public static void resetThread() {
		running = true;
	}

}
