/*
 * UbiCollab Service Domain Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.*;
import org.osgi.framework.*;
import no.ubicollab.osgi.service.domainmanager.*;

/**
 * The Class <code>Activator</code> contains the start and stop members, which is used
 * by the framework to start/activate and stop/deactivate this bundle (and thus the 
 * Service Domain Manager).
 * <p>
 * This bundle provides management functionality for services/bundles and will provide
 * a SOAP/Web-Service interface.
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class Activator implements BundleActivator {
	
	/** Handle to the Bundle Context. */
	private BundleContext bc = null;
	
	/** The Service Domain Manager implementation, implements <code>DomainManager</code>. */
	private DomainManagerImpl domainManager = null;
	
	
	
	/**
	 * This method is invoked by the framework to start the bundle and its service 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			this.bc = bc;
			Properties properties = new Properties();
			properties.put("Language", "English");
			properties.put("SOAP.service.name", GlobalSettings.SOAP_SERVICE_NAME);
			domainManager = new DomainManagerImpl(this.bc);
			bc.registerService(DomainManager.class.getName(),
					domainManager, properties);
			log.writeToLog("Service domain manager has started.", Log.INFO);
			System.out.println("Service Domain Manager has started.");
		} catch (Exception e) {
			log.writeToLog("Failed to start Service Domain Manager.", Log.ERROR);
			throw new BundleException("Failed to start Service Domain Manager", e);
		}
	}
	
	
	
	/**
	 * This method is invoked by the framework to stop the bundle and its service.
	 * All resources used is released to the garbage collector. 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			bc = null;
			domainManager = null;
			log.writeToLog("Service Domain Manager has stopped.", Log.INFO);
			System.out.println("Service Domain Manager has stopped.");
		} catch (Exception e) {
			log.writeToLog("Failed to stop Service Domain Manager.", Log.ERROR);
			throw new BundleException("Failed to stop Service Domain Manager", e);
		}
	}

}