/*
 * UbiCollab Service Domain Manager is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

/**
 * The <code>Service</code> Class is intended to store information about proxy 
 * services in UbiCollab.
 */
public class Service {
	
	/** The name of the service. */
	private String name;
	
	/** The service URI. */
	private String serviceURI;
	
	/** The description URI. */
	private String descriptionURI;
	
	/** The service type. */
	private String type;
	
	/** The UbiCollab Collaboration Space. */
	private String space;
	
	/** The ID for the UbiCollab Collaboration Space.. */
	private int spaceID;
	
	/** The bundle ID. */
	private long bundleID;
	
	/** The description for the service. */
	private String description;
	
	
	/**
	 * Constructor to create an empty Service object.
	 */
	public Service() {
		super();
	}
	
	
	/**
	 * Constructor for creating an Service object and setting the variables at the same time.
	 * 
	 * @param spaceID the space ID for the space this service is located in
	 * @param space the name of the space
	 * @param name the name
	 * @param bundleID the bundle ID
	 * @param serviceURI the service URI
	 * @param descriptionUri the URI pointing to the service description
	 * @param type the service type
	 * @param description a textual description of the service
	 */
	public Service(String name, String serviceURI, String space, int spaceID, long bundleID, String descriptionUri, String type, String description) {
		super();
		this.name = name;
		this.serviceURI = serviceURI;
		this.space = space;
		this.spaceID = spaceID;
		this.bundleID = bundleID;
		this.descriptionURI = descriptionUri;
		this.type = type;
		this.description = description;
	}
	
	
	/**
	 * Gets the XML represenatation for the service.
	 * 
	 * @param state the state of the service
	 * 
	 * @return the XML representing the service
	 */
	public String toXML(String state) {
		String xml = "<Service ";
		xml += "Id='" + bundleID + "' ";
		xml += "ServiceUri='" + serviceURI + "' ";
		xml += "Name='" + name + "' ";
		xml += "Space='" + space + "' ";
		xml += "SpaceID='" + spaceID + "' ";
		xml += "State='" + state + "' ";
		xml += "DescriptionUri='" + descriptionURI + "' ";
		xml += "Type='" + type + "' ";
		xml += "Description='" + description + "' />";
		return xml;
	}
	
	
	/**
	 * Gets the bundle ID attribute of the service.
	 * 
	 * @return the bundle ID attribute of the service
	 */
	public long getBundleID() {
		return bundleID;
	}
	
	
	/**
	 * Sets the bundle ID attribute of the service.
	 * 
	 * @param bundleID the bundle ID attribute of the service
	 */
	public void setBundleID(long bundleID) {
		this.bundleID = bundleID;
	}
	
	/**
	 * Gets the name attribute of the service.
	 * 
	 * @return the name attribute of the service
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the name attribute of the service.
	 * 
	 * @param name the name attribute of the service
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the service URI attribute of the service.
	 * 
	 * @return the service URI attribute of the service
	 */
	public String getServiceURI() {
		return serviceURI;
	}
	
	/**
	 * Sets the service URI attribute of the service.
	 * 
	 * @param serviceURI the service URI attribute of the service
	 */
	public void setServiceURI(String serviceURI) {
		this.serviceURI = serviceURI;
	}
	
	/**
	 * Gets the space attribute of the service.
	 * 
	 * @return the space attribute of the service
	 */
	public String getSpace() {
		return space;
	}
	
	/**
	 * Sets the space attribute of the service.
	 * 
	 * @param space the space attribute of the service
	 */
	public void setSpace(String space) {
		this.space = space;
	}
	
	/**
	 * Gets the space ID attribute of the service.
	 * 
	 * @return the space ID attribute of the service
	 */
	public int getSpaceID() {
		return spaceID;
	}
	
	/**
	 * Sets the space ID attribute of the service.
	 * 
	 * @param spaceID the space ID attribute of the service
	 */
	public void setSpaceID(int spaceID) {
		this.spaceID = spaceID;
	}


	/**
	 * Gets the description URI attribute of the service.
	 * 
	 * @return the description URI attribute of the service
	 */
	public String getDescriptionURI() {
		return descriptionURI;
	}


	/**
	 * Sets the description URI attribute of the service.
	 * 
	 * @param descriptionURI the description URI attribute of the service
	 */
	public void setDescriptionURI(String descriptionURI) {
		this.descriptionURI = descriptionURI;
	}


	/**
	 * Gets the type attribute of the service.
	 * 
	 * @return the type attribute of the service
	 */
	public String getType() {
		return type;
	}


	/**
	 * Sets the type attribute of the service.
	 * 
	 * @param type the type attribute of the service
	 */
	public void setType(String type) {
		this.type = type;
	}


	/**
	 * Gets the description attribute of the service.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}


	/**
	 * Sets the description attribute of the service.
	 * 
	 * @param description the description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	
}
