using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using ServiceManagementTool.serviceManager;
using ServiceManagementTool.UbiCollab;


namespace ServiceManagementTool
{


    #region Public Delegates releated to Service Discovery

    // delegates used to call ServiceGUI functions from discovery worker thread
    public delegate void DelegateAddServicesFound(String s);
    public delegate void DelegateSetServiceDiscoverySubscriptionID(int id);
    public delegate void DelegateSetServiceDiscoveryProtocols(String s);
    public delegate void DelegateThreadFinished();

    //delegates used to call ServiceGUI functions from CI
    public delegate void DelegateUpdateSpaces(ArrayList list);
    public delegate void DelegateCIThreadFinished();

    #endregion


    /// <summary>
    /// This is the main class in the Service Management Tool Solution. It provides the
    /// main GUI, connects to Web-services and will start the neccessary Threads.
    /// </summary>
    public partial class ServiceGUI : Form
    {

        #region class members

        //URL pointing to ubiHome
        private static string ubiHomeUrl = "http://ubicollab.pats.no/Service.asmx?WSDL";

        //Web reference to Service Domain Manager
        private DomainManagerImplService managerService;
        
        //Provides methods to parse XML received from Services.
        private XMLHandler xmlHandler;

        //private bool connected = false;
        private string ubiUserName;

        //This value is set when a service is selected in the Service Management Tab
        private long bundleIDtoEdit = -1;

        //Accesspoint for UbiCollab specific metohod regarding CS, CI and CI Publishing
        //private UbiConnector ubiConnector;

        //The worker thread that will provide Service Discovery connectivity
        Thread m_WorkerThread;

        //The worker thread that will provide Collaboration Space connectivity
        Thread s_WorkerThread;

        //Events used to stop discovery worker thread
        ManualResetEvent m_EventStopThread;
        ManualResetEvent m_EventThreadStopped;

        //Events used to stop CI worker thread
        ManualResetEvent s_EventStopThread;
        ManualResetEvent s_EventThreadStopped;

        // Delegate instances used to call GUI functions from discovery thread)
        public DelegateAddServicesFound m_DelegateAddServicesFound;
        public DelegateSetServiceDiscoverySubscriptionID m_DelegateSetServiceDiscoverySubscriptionID;
        public DelegateSetServiceDiscoveryProtocols m_DelegateSetServiceDiscoveryProtocols;
        public DelegateThreadFinished m_DelegateThreadFinished;

        //delegates used to call ServiceGUI functions from CI
        public DelegateUpdateSpaces s_DelegateUpdateSpaces;
        public DelegateCIThreadFinished s_DelegateCIThreadFinished;

        //The subscription id from the Service Discovery Manager
        private int serviceDiscoverySubscriptionID = -1;

        //The service discovery thread
        ServiceDiscoveryThread sdThread;

        //The collaboration space thread
        CollaborationSpaceThread csThread;

        #endregion


        /// <summary>
        /// This is the constructor which initializes the Service Management Tool.
        /// </summary>
        public ServiceGUI()
        {
            InitializeComponent();

            //initialize discovery delegates methods
            m_DelegateAddServicesFound = new DelegateAddServicesFound(this.AddServicesFound);
            m_DelegateSetServiceDiscoverySubscriptionID = new DelegateSetServiceDiscoverySubscriptionID(this.SetServiceDiscoverySubscriptionID);
            m_DelegateSetServiceDiscoveryProtocols = new DelegateSetServiceDiscoveryProtocols(this.SetServiceDiscoveryProtocols);
            m_DelegateThreadFinished = new DelegateThreadFinished(this.ThreadFinished);

            //initialise collab space delegates methods
            s_DelegateUpdateSpaces = new DelegateUpdateSpaces(this.updateSpaces);
            s_DelegateCIThreadFinished = new DelegateCIThreadFinished(this.ciThreadFinished);

            // initialize discovery events
            m_EventStopThread = new ManualResetEvent(false);
            m_EventThreadStopped = new ManualResetEvent(false);

            //initialise collab space events
            s_EventStopThread = new ManualResetEvent(false);
            s_EventThreadStopped = new ManualResetEvent(false);

            managerService = new DomainManagerImplService(); 
            
            xmlHandler = new XMLHandler();
            

            //TODO: Change this to read from config file.
            txtUbinodeURL.Text = "http://localhost:8080";
            txtLoginName.Text = "morten";

        }


        #region functions releated to service discovery

        /// <summary>
        /// Register a new service discovery request
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStartSearch_Click(object sender, EventArgs e)
        {
            if (sdThread != null)
            {
                sdThread.getServices(txtFindName.Text, txtFindType.Text,
                    txtFindLocation.Text, txtFindOwner.Text, txtFindDescription.Text);
                lwServicesFound.Items.Clear();
            }
        }


        /// <summary>
        /// Fires the service discovery thread
        /// </summary>
        private void startServiceDiscoveryThread()
        {
            // reset events
            m_EventStopThread.Reset();
            m_EventThreadStopped.Reset();

            // create worker thread instance
            m_WorkerThread = new Thread(new ThreadStart(this.serviceDiscoveryThreadFunction));
            m_WorkerThread.Name = "Service Discovery Thread";	// looks nice in Output window
            m_WorkerThread.Start();

            btnStartSearch.Enabled = true;
            txtFindName.Enabled = true;
            txtFindType.Enabled = true;
            txtFindLocation.Enabled = true;
            txtFindOwner.Enabled = true;
            txtFindDescription.Enabled = true;
            lwServicesFound.Enabled = true;
        }


        /// <summary>
        /// Starts the service discovery worker thread.
        /// Called indirectly from startServiceDiscoveryThread
        /// </summary>
        private void serviceDiscoveryThreadFunction()
        {
            sdThread = new ServiceDiscoveryThread(m_EventStopThread, m_EventThreadStopped, this, txtUbinodeURL.Text);
            sdThread.Run();
        }



        /// <summary>
        /// Stop service discovery worker thread if it is running.
        /// </summary>
        private void stopServiceDiscoveryThread()
        {
            Console.Out.WriteLine("Starter: STOP SERVICE DISCOVERY THREAD");
            if (m_WorkerThread != null && m_WorkerThread.IsAlive)  // thread is active
            {
                // set event "Stop"
                m_EventStopThread.Set();

                //Wait to see if thread will stop/finish
                while (m_WorkerThread.IsAlive)
                {
                    // Infinite wait cannot be used because the SD thread makes syncronous
                    // calls to main form, this will cause deadlock. Instead, wait for
                    // event some appropriate time (and give time to SD thread) and
                    // process events. These events may contain Invoke calls.
                    if (WaitHandle.WaitAll(
                        (new ManualResetEvent[] { m_EventThreadStopped }),
                        100,
                        true))
                    {
                        break;
                    }
                    Application.DoEvents();
                }
                txtFindName.Enabled = false;
                txtFindType.Enabled = false;
                txtFindLocation.Enabled = false;
                txtFindOwner.Enabled = false;
                txtFindDescription.Enabled = false;
                btnStartSearch.Enabled = false;
                lwServicesFound.Enabled = false;
            }
        }


        /// <summary>
        /// Updates the label which show which service discovery protocols are supported.
        /// </summary>
        /// <param name="s">A list of supported service discovery protocols</param>
        private void SetServiceDiscoveryProtocols(string s)
        {
            txtSDProtocols.Text = s;
            //lblSupportedProtocols.Text = "Supported Service Discovery Protocols: " + s;
        }



        /// <summary>
        /// Add services to the discovered services listview.
        /// Called from SD worker thread using delegate and Control.Invoke
        /// </summary>
        /// <param name="s">A xml-document with services</param>
        private void AddServicesFound(String s)
        {
            Console.Out.WriteLine(s);
            
            ListViewItem[] serviceListItems = xmlHandler.parseServiceFoundList(s);
            foreach (ListViewItem lvi in serviceListItems)
            {
                lwServicesFound.Items.Add(lvi);
            }
        }


        /// <summary>
        /// Sets the Service Discovery Subscription ID.
        /// Called from SD worker thread using delegate and Control.Invoke
        /// </summary>
        /// <param name="id">The new Service Discovery Subscription ID</param>
        private void SetServiceDiscoverySubscriptionID(int id)
        {
            serviceDiscoverySubscriptionID = id;
        }


        /// <summary>
        /// Set initial state of controls. Called from SD worker thread using 
        /// delegate and Control.Invoke. Tidy up.
        /// </summary>
        private void ThreadFinished()
        {
            btnStartSearch.Enabled = false;
        }




        #endregion




        #region functions related to collaboration spaces


        /// <summary>
        /// Fires the collaboration space thread
        /// </summary>
        private void startCollaborationSpaceThread()
        {
            // reset events
            s_EventStopThread.Reset();
            s_EventThreadStopped.Reset();

            // create worker thread instance
            s_WorkerThread = new Thread(new ThreadStart(this.collaborationSpaceFunction));
            s_WorkerThread.Name = "Collaboration Space Thread";	// looks nice in Profiler
            s_WorkerThread.Start();
        }


        /// <summary>
        /// Starts the collaboration space thread.
        /// Called indirectly from startCollaborationSpaceThread
        /// </summary>
        private void collaborationSpaceFunction()
        {
            csThread = new CollaborationSpaceThread(m_EventStopThread, m_EventThreadStopped, this,
                txtUbinodeURL.Text, ubiUserName);
            csThread.Run();
        }
        
        
        /// <summary>
        /// elegate method to be called from CS thread. Is called when a set of new 
        /// collaboration spaces have been fetched from collaboration manager.
        /// </summary>
        /// <param name="list">The ArrayList containing collaboration space data</param>
        private void updateSpaces(ArrayList list)
        {
            Console.Out.WriteLine("Receiving data from CS-thread...");

            cboEditServSpace.DataSource = list;
            cboEditServSpace.DisplayMember = "SpaceName";
            cboEditServSpace.ValueMember = "SpaceId";

        }


        /// <summary>
        /// Delegate method to be called from CS thread. Is called when the thread exits,
        /// and can be usefull for cleanup.
        /// </summary>
        private void ciThreadFinished()
        {
            Console.Out.WriteLine("CS-thread has stopped running...");
        }


        /// <summary>
        /// Stop service discovery worker thread if it is running.
        /// </summary>
        private void stopCollaborationSpaceThread()
        {
            if (s_WorkerThread != null && s_WorkerThread.IsAlive)  // thread is active
            {
                // set event "Stop"
                s_EventStopThread.Set();

                // wait when thread  will stop or finish
                while (s_WorkerThread.IsAlive)
                {
                    // Infinite wait cannot be used because the thread makes syncronous
                    // calls to main form, this will cause deadlock. Instead, wait for
                    // event some appropriate time (and give time to the thread) and
                    // process events. These events may contain Invoke calls.
                    if (WaitHandle.WaitAll(
                        (new ManualResetEvent[] { s_EventThreadStopped }),
                        100,
                        true))
                    {
                        break;
                    }

                    Application.DoEvents();
                }
            }
        }



        #endregion


        /// <summary>
        /// Fetches, format and update the service domain list
        /// </summary>
        /// <returns></returns>
        private int updateServiceList()
        {
            string sList = "";
            try
            {
                sList = managerService.getServiceList();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to connect to UbiNode. Please verify that the address is correct.\nMessage: " + ex.Message);
                return -1;
            }

            lwServiceView.BeginUpdate();
            lwServiceView.Items.Clear();

            ListViewItem[] serviceListItems = xmlHandler.parseServiceList(sList);
            foreach (ListViewItem lvi in serviceListItems)
            {
                if (lvi.SubItems[5].Text.Equals("Started"))
                    lvi.ForeColor = Color.Green;
                else if (lvi.SubItems[5].Text.Equals("Stopped"))
                    lvi.ForeColor = Color.Blue;
                else
                    lvi.ForeColor = Color.Red;
                lwServiceView.Items.Add(lvi);
            }
            lwServiceView.EndUpdate();
            return serviceListItems.Length;
        }
        
        

        private void btnConnectToUbinode_Click(object sender, EventArgs e)
        {
            int result = 0;

            if (txtUbinodeURL.Text.Length > 2)
            {
                ubiUserName = txtLoginName.Text;
                managerService.Url = txtUbinodeURL.Text + "/axis/services/serviceManager";

                if (updateServiceList() != -1)
                {
                    //Enable buttons and stuff.
                    grpServList.Enabled = true;
                    btnUpdateLog.Enabled = true;
                    txtLoginName.Enabled = false;
                    txtUbinodeURL.Enabled = false;

                    //Start service discovery Thread
                    startServiceDiscoveryThread();

                    //Start collaboration space thread
                    startCollaborationSpaceThread();

                    //Enable log reading tab
                    enableLogReading();

                    //Set connect buttons status
                    btnConnectToUbinode.Enabled = false;
                    btnDisconnect.Enabled = true;
                }
            }
        }


        #region functions related to Log tabular

        /// <summary>
        /// Format the log tabular to enable log reading. Must be called after
        /// it is connected to the services.
        /// </summary>
        private void enableLogReading()
        {
            LogManager lm = new LogManager();
            cboLogLevel.Enabled = true;
            cboLogLevel.DataSource = lm.LogLevelList;
            cboLogLevel.DisplayMember = "LevelName";
            cboLogLevel.ValueMember = "LevelValue";
            cboLogLevel.DropDownStyle = ComboBoxStyle.DropDownList;
            cboLogItems.Enabled = true;
            cboLogItems.DataSource = lm.LogDisplayCount;
            cboLogItems.DisplayMember = "LevelName";
            cboLogItems.ValueMember = "LevelValue";
            cboLogItems.DropDownStyle = ComboBoxStyle.DropDownList;
            cboLogSource.Enabled = true;
            cboLogSource.DataSource = lm.LogSource;
            cboLogSource.DisplayMember = "LevelName";
            cboLogSource.ValueMember = "LevelValue";
            cboLogSource.DropDownStyle = ComboBoxStyle.DropDownList;
            btnUpdateLog.Enabled = true;
            lwLogView.Enabled = true;
        }

        /// <summary>
        /// Format the log tabular to disable log reading. Must be called after
        /// a diconnect is performed.
        /// </summary>
        private void disableLogReading()
        {
            cboLogLevel.Enabled = false;
            cboLogItems.Enabled = false;
            cboLogSource.Enabled = false;
            btnUpdateLog.Enabled = false;
            lwLogView.Enabled = false;
        }

        /// <summary>
        /// Event handler for buttonClick on the log update button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpdateLog_Click(object sender, EventArgs e)
        {
            int logLevel = Convert.ToInt16(cboLogLevel.SelectedValue);
            string logLevelName = cboLogLevel.Text;
            int logItemCount = Convert.ToInt16(cboLogItems.SelectedValue);
            string xmlLog;

            if (Convert.ToInt16(cboLogSource.SelectedValue) == 1)
            {
                xmlLog = managerService.getLogItems(logLevel, logItemCount);
            }
            else
                xmlLog = sdThread.getLogItems(logLevel, logItemCount);

            lwLogView.Items.Clear();
            ListViewItem[] logListItems = xmlHandler.parseLog(xmlLog);
            foreach (ListViewItem lvi in logListItems)
                lwLogView.Items.Add(lvi);
        }

        #endregion


        private void lwServiceView_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListViewItem sItem = (ListViewItem)((ListView)sender).FocusedItem;

            //SelectedIndexChanged is fired both when an item is unselected and when a new 
            //is selected. Only act when new are selected. 
            if (sItem != null)
            {
                btnServEdit.Enabled = true;
                //MessageBox.Show(sItem.SubItems[2].Text);
            }
        }

        private void btnServEdit_Click(object sender, EventArgs e)
        {
            if (lwServiceView.FocusedItem != null)
            {
                txtEditServName.Text = lwServiceView.FocusedItem.SubItems[2].Text;
                txtEditServDescription.Text = lwServiceView.FocusedItem.SubItems[3].Text;
                
                //TODO:Fix
                cboEditServSpace.Text = lwServiceView.FocusedItem.SubItems[4].Text;
                
                bundleIDtoEdit = (long) Convert.ToInt16(lwServiceView.FocusedItem.SubItems[0].Text);
                grpServList.Enabled = false;
                grpServEdit.Enabled = true;
            }
            else
                MessageBox.Show("You must select a service before you can edit it.");
            
        }

        private void btnServEditConfirm_Click(object sender, EventArgs e)
        {
            if (txtEditServName.Text.Length > 2)
            {
                int spaceIndex = Convert.ToInt16(cboEditServSpace.SelectedValue);
                //string spaceName;
                //cboEditServSpace.SelectedValue;

                if (managerService.updateServiceProperties(bundleIDtoEdit, txtEditServName.Text, cboEditServSpace.Text, spaceIndex, txtEditServType.Text, txtEditServDescription.Text))
                {
                    txtEditServName.Text = "";
                    txtEditServDescription.Text = "";
                    //cboEditServSpace.Text = "";
                    grpServList.Enabled = true;
                    updateServiceList();
                    grpServEdit.Enabled = false;
                }
                else
                    MessageBox.Show("Service update failed, check log for details.");
            }
            else
                MessageBox.Show("You must give the service a name langer then two characters.");
        }



        private void btnServEditCancel_Click(object sender, EventArgs e)
        {
            txtEditServName.Text = "";
            //txtServEditSpace.Text = "";
            txtEditServDescription.Text = "";
            grpServList.Enabled = true;
            grpServEdit.Enabled = false;
        }

        
        /// <summary>
        /// Handler for the Service Stop Button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnServStop_Click(object sender, EventArgs e)
        {
            //int index = -1;
            if (lwServiceView.FocusedItem != null)
            {
                int index = lwServiceView.SelectedItems[0].Index;
                if (!managerService.stopService(Convert.ToInt32(lwServiceView.FocusedItem.SubItems[0].Text)))
                    MessageBox.Show("Unable to stop service. Check log for details.");
                else
                    updateServiceList();
                lwServiceView.Items[index].Selected = true;
                lwServiceView.Items[index].Focused = true;
                lwServiceView.Refresh();
                lwServiceView.Focus();
            }
            else
                MessageBox.Show("You must select a service before you can stop it.");
        }

        
        /// <summary>
        /// Handler for the Service Start button.
        /// </summary>
        private void btnServStart_Click(object sender, EventArgs e)
        {
            if (lwServiceView.FocusedItem != null)
            {
                int index = lwServiceView.SelectedItems[0].Index;
                if (!managerService.startService(Convert.ToInt16(lwServiceView.FocusedItem.SubItems[0].Text)))
                    MessageBox.Show("Unable to start service. Check log for details.");
                else
                    updateServiceList();
                lwServiceView.Items[index].Selected = true;
                lwServiceView.Items[index].Focused = true;
                lwServiceView.Refresh();
                lwServiceView.Focus();
            }
            else
                MessageBox.Show("You must select a service before you can start it.");
        }

        
        
        /// <summary>
        /// Opens a new Dialog with the option to install a service
        /// </summary>
        private void manualInstallToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManualServiceInstall msi = new ManualServiceInstall(managerService);
            msi.ShowDialog();
            updateServiceList();
        }


        /// <summary>
        /// Clean up when the application close, stop threads still running.
        /// </summary>
        private void ServiceGUI_FormClosed(object sender, FormClosedEventArgs e)
        {
            stopServiceDiscoveryThread();
            stopCollaborationSpaceThread();
        }

        
        /// <summary>
        /// Handler for the Publish Assistant button Click. Will open a new dialog to handle
        /// service publishing.
        /// </summary>
        private void btnServPublish_Click(object sender, EventArgs e)
        {
            if (lwServiceView.FocusedItem != null)
            {
                int id = Convert.ToInt16(lwServiceView.FocusedItem.SubItems[0].Text);
                string name = lwServiceView.FocusedItem.SubItems[2].Text;

                PublishAssistant pa = new PublishAssistant(txtLoginName.Text, id, name, ubiHomeUrl);
                pa.ShowDialog();
            }
            else
                MessageBox.Show("You must select a service, before you can start the publish assistant.");
        }


        /// <summary>
        /// Handler for the disconnect button
        /// </summary>
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            //Stop the working threads and dispose Service Domain Manager object.
            stopServiceDiscoveryThread();
            stopCollaborationSpaceThread();
            managerService.Dispose();

            //Disable and enable GUI elements.
            disableLogReading(); //Will disable the log-related controls
            grpServList.Enabled = false;
            btnConnectToUbinode.Enabled = true;
            btnDisconnect.Enabled = false;
            txtLoginName.Enabled = true;
            txtUbinodeURL.Enabled = true;
        }


        /// <summary>
        /// Handler for menu About click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About a = new About();
            a.ShowDialog();
        }


        /// <summary>
        /// Handler for the event thrown when a user double-clicks on a discovered service.
        /// </summary>
        private void lwServicesFound_DoubleClick(object sender, EventArgs e)
        {
            if (lwServicesFound.FocusedItem != null)
            {
                ListViewItem lvi = lwServicesFound.FocusedItem;
                DisplayServiceInformation dsi = new DisplayServiceInformation(lvi.SubItems[0].Text,
                    lvi.SubItems[1].Text, lvi.SubItems[2].Text, lvi.SubItems[3].Text,
                    lvi.SubItems[4].Text, lvi.SubItems[5].Text, lvi.SubItems[6].Text,
                    lvi.SubItems[7].Text, managerService);
                dsi.ShowDialog();
                updateServiceList();
            }
                
        }


        /// <summary>
        /// Handler for the delete service button click.
        /// </summary>
        private void btnServUnistall_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Please confirm that you really wish to uninstall service.", "Confirm delete", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {

                if (lwServiceView.FocusedItem != null)
                {
                    if (!managerService.removeService(Convert.ToInt32(lwServiceView.FocusedItem.SubItems[0].Text)))
                        MessageBox.Show("Unable to uninstall service " + lwServiceView.FocusedItem.SubItems[0].Text + ". Check log for details.");
                    else
                        updateServiceList();
                }
                else
                    MessageBox.Show("You must select a service before you can uninstall it.");
            }
            else
                MessageBox.Show("Ok, just be a wimp then.");
        }




    }
}