using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections;
using ServiceManagementTool.ubiHome;
using ServiceManagementTool.CSManager;
using ServiceManagementTool.CIManager;
using System.Windows.Forms;
using System.Web.Services.Protocols;

namespace ServiceManagementTool.UbiCollab
{

    class UbiConnector
    {
        /* Reference to the ubiHome service */
        private Service ubiHome;

        /* Reference to the CS Manager */
        private CSManagerImplService csManager;

        /* Reference to the CI Manager */
        private CIManagerImplService ciManager;

        /* Is CS Manager connected? */
        private bool csManagerConnected = false;

        /* Is CI Manager connected? */
        private bool ciManagerConnected = false;

        /* Is ubiHome connected? */
        private bool ubiHomeConnected = false;

        /* List of existing Collaboration Spaces */
        private ArrayList collabSpaceList = new ArrayList();

        /* List of existing Collaboration Instances */
        private ArrayList collabInstanceList = new ArrayList();

        /* List of published services */
        private ArrayList publishedServices = new ArrayList();

        /* Name of the user to log on to the services */
        private string userName;
        
        
        public UbiConnector(string ubiHomeUrl, string hostUrl, string userName)
        {
            ubiHomeUrl = "http://ubicollab.pats.no/Service.asmx?WSDL";
            hostUrl = "http://localhost:8080/axis/services/CSManager?wsdl";

            this.userName = userName;

            //Login to Collaboration Space Manager
            try
            {
                csManager = new CSManagerImplService();
                csManager.Url = hostUrl;
                csManager.login(userName);
                csManagerConnected = true;

            }
            catch (SoapException e)
            {
                MessageBox.Show("Unable to connect to Collaboration Space Manager (" + hostUrl + ").\n Message: " +e.Message );
                Console.Out.WriteLine(e.StackTrace);
            }

            //Login to Collaboration Instance Manager
            try
            {
                ciManager = new CIManagerImplService();
                ciManager.Url = hostUrl;
                ciManager.login(userName);
                ciManagerConnected = true;

            }
            catch (SoapException e)
            {
                MessageBox.Show("Unable to connect to Collaboration Instance Manager (" + hostUrl + ").\n Message: " + e.Message);
                Console.Out.WriteLine(e.StackTrace);
            }

            //Login to UbiHome
            ubiHome = new Service();
            ubiHomeConnected = true;

        }



        public void getCollaborationSpaces(ComboBox cBox)
        {
            if ( csManagerConnected )
            {
                string collabSpaces =  csManager.getCsString();
                string[] spaceItems;

                foreach (string collabSpaceItem in collabSpaces.Split(char.Parse("\n")))
                {
                    spaceItems = collabSpaceItem.Split(char.Parse("_"));
                    if (spaceItems.Length > 2)
                    {
                        collabSpaceList.Add(new CollabSpace(spaceItems[0], spaceItems[1]));
                    }
                }

                cBox.DataSource = collabSpaceList;
                cBox.DisplayMember = "SpaceName";
                cBox.ValueMember = "SpaceId";

            }
            else
                MessageBox.Show("You must first be connected to Collaboration Space Manager.");
        }

        
        
        public void getCollaborationInstances()
        {
            if (ciManagerConnected)
            {
                string ciListXML = ubiHome.getAll(userName);
                string ccid = "";
                string name = "";

                // Read xml-string
                XmlTextReader textReader = new XmlTextReader(new System.IO.StringReader(ciListXML));
                
                // Read through all elements in string.
                while (textReader.Read())
                {
                    XmlNodeType nType = textReader.NodeType; 

                    if (textReader.NodeType == XmlNodeType.Element && textReader.Name == "CIID")
                        ccid = textReader.ReadElementContentAsString();

                    if (nType == XmlNodeType.Element && textReader.Name == "name")
                        name = textReader.ReadElementContentAsString();

                    if (ccid != "" && name != "")
                    {
                        collabInstanceList.Add(new CollabInstance(ccid, name));
                        ccid ="";
                        name = "";
                    }
                }

            }
            else
                MessageBox.Show("You must first be connected to Collaboration Instance Manager.");
            updatePublishInformation();
        }



        private void updatePublishInformation()
        {
            if (ubiHomeConnected)
            {
                string tmp = ubiHome.getPublishedServicesUser(userName);

                Console.Out.WriteLine("Published services=" + tmp);

                //publishedServices


            }
            else
                MessageBox.Show("Unable to connect to UbiHome.");
        }



        public string getPublishedToInstances(int serviceID)
        {


            return null;
        }


    }
}
