using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace ServiceManagementTool
{
    class XMLHandler
    {


        /// <summary>
        /// Method to convert a xml-document (from a string) to the format needed for insertion in a ListView
        /// </summary>
        /// <param name="s">Must be a valid xml-formated list containing the services</param>
        /// <returns>The service-list formated and ready to be inserted in a ListView control</returns>
        public ListViewItem[] parseServiceList(String s)
        {
            //Read elements into dynamic items[], then transfer to serviceList[]
            ArrayList items = new ArrayList();
            ListViewItem[] serviceList = null;

            ListViewItem lvi;
            ListViewItem.ListViewSubItem lvsi;
            
            XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

            while (reader.Read())
            {
                // Read the title attribute 					
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Service" && reader.AttributeCount > 0)
                {
                    lvi = new ListViewItem();
                    lvi.Text = reader.GetAttribute("Id");
                    lvi.ImageIndex = 1;
                    lvi.Tag = reader.GetAttribute("Id");

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Url");
                    lvsi.Tag = "Url";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Name");
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Description");
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Space");
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("State");
                    lvsi.Tag = "State";
                    lvi.SubItems.Add(lvsi);

                    items.Add(lvi);
                }

            }
            
            serviceList = new ListViewItem[items.Count];
            int index = 0;
            foreach (Object o in items)
            {
                serviceList[index] = (ListViewItem)o;
                index++;
            }

            return serviceList;
        }



        public ListViewItem[] parseLog(String s)
        {
            //Read elements into the dynamic items[], then transfer to serviceList[]
            ArrayList items = new ArrayList();
            ListViewItem[] logList = null;

            ListViewItem lvi;
            ListViewItem.ListViewSubItem lvsi;

            try
            {
                XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

                while (reader.Read())
                {
                    // Read the title attribute 					
                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "Logitem" && reader.AttributeCount > 0)
                    {
                        lvi = new ListViewItem();
                        lvi.Text = reader.GetAttribute("Date");
                        lvi.ImageIndex = 1;
                        //lvi.Tag = reader.GetAttribute("Id");

                        //lvsi = new ListViewItem.ListViewSubItem();
                        //lvsi.Text = reader.GetAttribute("Date");
                        //lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Time");
                        lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Level");
                        lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Message");
                        lvi.SubItems.Add(lvsi);

                        items.Add(lvi);
                    }

                }
            }
            catch (XmlException xmlE)
            {
                //alert("Unable to parse response from Service Domain Manager. Aborting.");
                Console.Out.WriteLine(xmlE.StackTrace);
            }

            logList = new ListViewItem[items.Count];
            int index = 0;
            foreach (Object o in items)
            {
                logList[index] = (ListViewItem)o;
                index++;
            }

            return logList;
        }


        /// <summary>
        /// Method that parses an xml-document containing the result from
        /// a WS-call to discoveryManager#getSDPlugins.
        /// </summary>
        /// <param name="s">The string to parse</param>
        /// <returns>A comma separated list with protocol names</returns>
        public string parseSDProtocolList(String s)
        {
            string protList = "";
            try
            {
                XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

                while (reader.Read())
                {
                    // Read the title attribute 					
                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "Plugin" && reader.AttributeCount > 0)
                    {
                        if (protList.Length > 1)
                            protList += ", ";
                        protList += reader.GetAttribute("Name");
                    }
                }
            }
            catch (XmlException xmlE)
            {
                Console.Out.WriteLine(xmlE.StackTrace);
            }
            return protList;
        }



        /// <summary>
        /// Method to convert a xml-document (from a string) to the format needed for insertion in a ListView
        /// </summary>
        /// <param name="s">Must be a valid xml-formated list containing the services</param>
        /// <returns>The service-list formated and ready to be inserted in a ListView control</returns>
        public ListViewItem[] parseServiceFoundList(String s)
        {
            //Read elements into dynamic items[], then transfer to serviceList[]
            ArrayList items = new ArrayList();
            ListViewItem[] serviceList = null;

            ListViewItem lvi;
            ListViewItem.ListViewSubItem lvsi;

            XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

            while (reader.Read())
            {
                // Read the title attribute 					
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Service" && reader.AttributeCount > 0)
                {
                    lvi = new ListViewItem();
                    lvi.Text = reader.GetAttribute("ServiceUri");
                    lvi.ImageIndex = 1;
                    lvi.Tag = reader.GetAttribute("ServiceUri");

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("DescriptionUri");
                    lvsi.Tag = "DescriptionUri";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Name");
                    lvsi.Tag = "Name";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Type");
                    lvsi.Tag = "Type";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Location");
                    lvsi.Tag = "Location";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Owner");
                    lvsi.Tag = "Owner";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Description");
                    lvsi.Tag = "Description";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Protocol");
                    lvsi.Tag = "Protocol";
                    lvi.SubItems.Add(lvsi);


                    items.Add(lvi);
                }

            }

            serviceList = new ListViewItem[items.Count];
            int index = 0;
            foreach (Object o in items)
            {
                serviceList[index] = (ListViewItem)o;
                index++;
            }

            return serviceList;
        }
    
    
    }
}
