/*
 * UbiCollab Service Registry is developed by Kim-Steve Johansen as part of 
 * a master thesis at The Norwegian University of Science and Technology (NTNU). 
 * 
 * UbiCollab is an open source software project available at http://sourceforge.net/projects/ubicollab 
 * and is distributable under the Apache License v2.0 (http://www.apache.org/licenses/).
 * 
 * For more information, please contact Kim-Steve Johansen at kimsteve@stud.ntnu.no
 */
package no.ubicollab.osgi;

import java.util.*;
import org.osgi.framework.*;
import no.ubicollab.osgi.service.registry.*;

/**
 * The Class Activator contains the start method, which is the method called
 * by the framework to start/activate this bundle (and then the registry service).
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class Activator implements BundleActivator {

	/** Handle to the Bundle Context. */
	//private BundleContext bc = null;

	/** The service registry. */
	private ServiceRegistryImpl serviceRegistry = null;

	/** The database connection object. */
	private DBConnection dbConnection = null;
	
	
	
	/**
	 * This method is invoked by the framework to start the bundle and its service 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			//this.bc = bc;
			Properties properties = new Properties();
			properties.put("Language", "English");
			properties.put("SOAP.service.name", GlobalSettings.SOAP_SERVICE_NAME);
			dbConnection = new DBConnection();
			serviceRegistry = new ServiceRegistryImpl(dbConnection);
			bc.registerService(ServiceRegistry.class.getName(),
					serviceRegistry, properties);
			System.out.println("Service Registry has started");
			log.writeToLog("Service Registry has started", Log.INFO);
		} catch (Exception e) {
			log.writeToLog("Unabe to start Service Registry. Message:"
					+ e.getMessage(), Log.ERROR);
			throw new BundleException("Failed to start Service Registry", e);
		}
	}
	
	
	
	/**
	 * This method is invoked by the framework to stop the bundle and its service.
	 * All resources used is released to the garbage collector. 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			dbConnection.disconnectFromDatabase();
			serviceRegistry = null;
			dbConnection = null;
			log.writeToLog("Service Registry has stopped.", Log.INFO);
			System.out.println("Service Registry was stopped");
		} catch (Exception e) {
			log.writeToLog("Failed to stop Service Registry service and release resources. " +
					"Message:" + e.getMessage(), Log.ERROR);
			throw new BundleException("Failed to stop Service Registry", e);
		}
	}

}