using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using ServiceRegistryTool.ServiceRegistry;
using System.Web.Services;
using System.Text.RegularExpressions;

namespace ServiceRegistryTool
{
    public partial class Form1 : Form
    {
        int selectedService = -1;

        ServiceRegistryImplService serviceRegistry = null;

        XMLHandler xmlHandler = new XMLHandler();

        public Form1()
        {
            InitializeComponent();

            txtServiceRegistryURL.Text = "http://localhost:8080/axis/services/serviceRegistry";

        }



        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About a = new About();
            a.ShowDialog();
        }



        private void btnConnect_Click(object sender, EventArgs e)
        {
            //Add http:// to url if missing
            System.Globalization.CompareInfo cmpUrl = System.Globalization.CultureInfo.InvariantCulture.CompareInfo;
            if (cmpUrl.IsPrefix(txtServiceRegistryURL.Text, "http://") == false)
            {
                txtServiceRegistryURL.Text = "http://" + txtServiceRegistryURL.Text;
            }
            
            //
            if ( validateUrl(txtServiceRegistryURL.Text) )
            {
                try
                {
                    serviceRegistry = new ServiceRegistryImplService();
                    serviceRegistry.Url = txtServiceRegistryURL.Text;
                    serviceRegistry.getServiceDetails(-1);
                    
                    //Enable group boxes
                    groupAdd.Enabled = true;
                    groupDiscover.Enabled = true;

                    //Disable edit boxes
                    btnEditService.Enabled = false;
                    btnDeleteService.Enabled = false;
                }
                catch (Exception ex)
                {
                    groupAdd.Enabled = false;
                    groupDiscover.Enabled = false;
                    MessageBox.Show("Could not connect to Service Registry. \r\nMessage:" + ex.Message);
                    Console.Out.WriteLine(ex.StackTrace);
                }
            }
            else
                MessageBox.Show("Service Registry URL is not a valid URL, please try again.");
        }

        private void btnAddService_Click(object sender, EventArgs e)
        {
            try
            {
                if (serviceRegistry.addService(txtAddServiceURI.Text, txtAddDescriptionURI.Text, txtAddName.Text, txtAddType.Text, txtAddLocation.Text, txtAddOwner.Text, txtAddDescription.Text))
                {
                    txtAddServiceURI.Text = "";
                    txtAddDescriptionURI.Text = "";
                    txtAddName.Text = "";
                    txtAddType.Text = "";
                    txtAddLocation.Text = "";
                    txtAddOwner.Text = "";
                    txtAddDescription.Text = "";
                    MessageBox.Show("Service was successfully added.");
                }
                else
                    MessageBox.Show("Service could not be added to Service Registry.\r\nCheck Service Registry log for details.");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Service could not be added to Service Registry. \r\nMessage:" + ex.Message);
                Console.Out.WriteLine(ex.StackTrace);
            }
        }

        private void btnStartSearch_Click(object sender, EventArgs e)
        {
            try
            {
                lwServices.Items.Clear();
                string sList = serviceRegistry.getServices(txtFindName.Text, txtFindType.Text, txtFindLocation.Text, txtFindOwner.Text, txtFindDescription.Text);
                Console.Out.WriteLine(sList);
                ListViewItem[] lviServiceList = xmlHandler.parseServiceList(sList);
                foreach (ListViewItem lvi in lviServiceList)
                {
                    lwServices.Items.Add(lvi);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Service could not be added to Service Registry. \r\nMessage:" + ex.Message);
                Console.Out.WriteLine(ex.StackTrace);
            }
        }




        /// <summary>
        /// A help method used to validate URL's
        /// </summary>
        /// <param name="url">The url to verify</param>
        /// <returns>True if valid, false othervise</returns>
        private bool validateUrl(string url)
        {
           return (true);
        }



        private void lwServices_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            ListViewItem sItem = (ListViewItem)((ListView)sender).FocusedItem;
            
            //SelectedIndexChanged is fired both when an item is unselected and when a new 
            //is selected. Only act when new are selected. 
            if (e.IsSelected)
            {
                btnEditService.Enabled = true;
                btnDeleteService.Enabled = true;
                selectedService = Int32.Parse(sItem.SubItems[0].Text);
                //MessageBox.Show("si0=" + sItem.SubItems[0].Text + " si1=" + sItem.SubItems[1].Text);
            }
            else
            {
                btnEditService.Enabled = false;
                btnDeleteService.Enabled = false;
                selectedService = -1;
            }
                
        }

        private void btnEditService_Click(object sender, EventArgs e)
        {
            if (lwServices.FocusedItem != null)
            {
                ListViewItem sItem = lwServices.FocusedItem;
                UpdateServiceInfo usi = new UpdateServiceInfo(serviceRegistry, selectedService,
                    sItem.SubItems[1].Text, sItem.SubItems[2].Text, sItem.SubItems[3].Text,
                    sItem.SubItems[4].Text, sItem.SubItems[5].Text, sItem.SubItems[6].Text,
                    sItem.SubItems[7].Text);
                usi.ShowDialog();
            }
        }

        private void btnDeleteService_Click(object sender, EventArgs e)
        {
            try
            {
                if (serviceRegistry.deleteService(selectedService))
                    MessageBox.Show("Service was deleted");
                else
                    MessageBox.Show("Failed to delete service. Consult Service Registry log for details.");
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(ex.StackTrace);
            }
        }


    }
}