using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Xml;

namespace ServiceRegistryTool
{
    
    /// <summary>
    /// Class containing the methods needed to parse XML received from Service Registry
    /// </summary>
    class XMLHandler
    {

        /// <summary>
        /// Method to convert a xml-document (from a string) to the format needed for insertion in a ListView
        /// </summary>
        /// <param name="s">Must be a valid xml-formated list containing the services</param>
        /// <returns>The service-list formated and ready to be inserted in a ListView control</returns>
        public ListViewItem[] parseServiceList(String s)
        {
            //Read elements into dynamic items[], then transfer to serviceList[]
            ArrayList items = new ArrayList();
            ListViewItem[] serviceList = null;

            ListViewItem lvi;
            ListViewItem.ListViewSubItem lvsi;

            XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

            while (reader.Read())
            {
                // Read the title attribute 					
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Service" && reader.AttributeCount > 0)
                {
                    lvi = new ListViewItem();
                    lvi.Text = reader.GetAttribute("ID");
                    lvi.ImageIndex = 1;
                    lvi.Tag = reader.GetAttribute("ID");

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("ServiceUri");
                    lvsi.Tag = "ServiceUri";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("DescriptionUri");
                    lvsi.Tag = "DescriptionUri";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Name");
                    lvsi.Tag = "Name";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Type");
                    lvsi.Tag = "Type";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Location");
                    lvsi.Tag = "Location";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Owner");
                    lvsi.Tag = "Owner";
                    lvi.SubItems.Add(lvsi);

                    lvsi = new ListViewItem.ListViewSubItem();
                    lvsi.Text = reader.GetAttribute("Description");
                    lvsi.Tag = "Description";
                    lvi.SubItems.Add(lvsi);

                    items.Add(lvi);
                }

            }

            serviceList = new ListViewItem[items.Count];
            int index = 0;
            foreach (Object o in items)
            {
                serviceList[index] = (ListViewItem)o;
                index++;
            }

            return serviceList;
        }



        public ListViewItem[] parseLog(String s)
        {


            //Read elements into the dynamic items[], then transfer to serviceList[]
            ArrayList items = new ArrayList();
            ListViewItem[] logList = null;

            ListViewItem lvi;
            ListViewItem.ListViewSubItem lvsi;

            try
            {
                XmlTextReader reader = new XmlTextReader(new System.IO.StringReader(s));

                while (reader.Read())
                {
                    // Read the title attribute 					
                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "Logitem" && reader.AttributeCount > 0)
                    {
                        lvi = new ListViewItem();
                        lvi.Text = reader.GetAttribute("Date");
                        lvi.ImageIndex = 1;
                        //lvi.Tag = reader.GetAttribute("Id");

                        //lvsi = new ListViewItem.ListViewSubItem();
                        //lvsi.Text = reader.GetAttribute("Date");
                        //lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Time");
                        lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Level");
                        lvi.SubItems.Add(lvsi);

                        lvsi = new ListViewItem.ListViewSubItem();
                        lvsi.Text = reader.GetAttribute("Message");
                        lvi.SubItems.Add(lvsi);

                        items.Add(lvi);
                    }

                }
            }
            catch (XmlException xmlE)
            {
                //alert("Unable to parse response from Service Domain Manager. Aborting.");
                Console.Out.WriteLine(xmlE.StackTrace);
            }

            logList = new ListViewItem[items.Count];
            int index = 0;
            foreach (Object o in items)
            {
                logList[index] = (ListViewItem)o;
                index++;
            }

            return logList;
        }



    }
}
