using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace X10Client
{
    /// <summary>
    /// This class provides persistant storeage of X10 devices, by serializing a
    /// device list to and from file using .NET xml-Synchronizaiton
    /// </summary>
    class DeviceStore
    {
        // Create a file object
        public static readonly string fileName = "X10_devices.xml";


        
        /// <summary>
        /// This method will deserialize a stored List, formatted as a xml-document, 
        /// from a file.
        /// </summary>
        /// <returns>The deserialized list</returns>
        public static List<X10Device> LoadDevices()
        {
            List<X10Device> ret = new List<X10Device>();
            if (System.IO.File.Exists(fileName))
            {
                System.Xml.Serialization.XmlSerializer xs = new System.Xml.Serialization.XmlSerializer(typeof(List<X10Device>));
                using (System.IO.StreamReader sr = new System.IO.StreamReader(fileName))
                {
                    ret = xs.Deserialize(sr) as List<X10Device>;
                }
            }

            return ret;
        }



        /// <summary>
        /// This method will serialise the provided List to a file, as an xml-document.
        /// </summary>
        /// <param name="list">The list to serialize</param>
        public static void SaveDevices(List<X10Device> list)
        {
            System.Xml.Serialization.XmlSerializer xs = new System.Xml.Serialization.XmlSerializer(typeof(List<X10Device>));
            using (System.IO.StreamWriter sw = new System.IO.StreamWriter(fileName))
            {
                xs.Serialize(sw, list);
            }
        }
    }
}
