package org.ubicollab.X10Proxy;

import java.util.*;
import org.osgi.framework.*;
import org.ubicollab.service.X10Proxy.*;


/**
 * The Class <code>Activator</code> contains the start and stop methods, which 
 * is used by the framework to start/activate and stop/deactivate this bundle 
 * (and thus the X10 Proxy Service).
 * 
 * @author Kim-Steve Johansen
 * @version 0.1
 */
public class Activator implements BundleActivator {

	/** The x10 proxy implementation. */
	private X10ProxyImpl x10Proxy = null;

	
	
	/**
	 * This method is invoked by the framework to start the bundle and its service 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		try {
			Properties properties = new Properties();
			properties.put("Language", "English");
			properties.put("SOAP.service.name", GlobalSettings.SOAP_SERVICE_NAME);
			properties.put("UbiCollab.service-proxy.name", GlobalSettings.SOAP_SERVICE_NAME);
			x10Proxy = new X10ProxyImpl();
			bc.registerService(X10Proxy.class.getName(),
					x10Proxy, properties);
			System.out.println("X10 Proxy has started");
		} catch (Exception e) {
			throw new BundleException("Failed to start X10 Proxy", e);
		}
	}
	
	
	
	/**
	 * This method is invoked by the framework to stop the bundle and its service.
	 * All resources used is released to the garbage collector. 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		try {
			x10Proxy.stopSerialConnection();
			System.out.println("X10 Proxy has stopped");
		} catch (Exception e) {
			throw new BundleException("Failed to stop X10 Proxy", e);
		}
	}

}