package org.ubicollab.X10Proxy;

import org.ubicollab.service.X10Proxy.X10Proxy;


/**
 * The Class <code>X10ProxyImpl</code> implements the methods neccessary for
 * a minimal x10 implementation.
 * 
 * @author Kim-Steve Johansen
 * @version 0.1
 */
public class X10ProxyImpl implements X10Proxy {
	
	/** The implementation of the neccessary x10 methods. */
	private X10Impl x10Controller;

	/** <code>true</code> if a serial connection is open, 
	 * <code>false</code> othervise. */
	private boolean connected = false;
	
	
	
	/**
	 * This is an empty constructor.
	 */
	public X10ProxyImpl() {
	}
	
	
	
	/**
	 * Send a on/off command to an X10 device identified by <code>house</code>
	 * and <code>unit</code>.
	 * 
	 * @see org.ubicollab.service.X10Proxy.X10Proxy#sendCommand(java.lang.String, int, boolean)
	 */
	public void sendCommand(String house, int unit, boolean turnOn) {
		if ( connected ) {
			char houseUnit = house.charAt(0);
			if (turnOn)
				x10Controller.turnOnDevice(houseUnit, unit);
			else
				x10Controller.turnOffDevice(houseUnit, unit);
		}
	}

	
	
	/**
	 * Start serial connection to a CM11A X10 control device on the given
	 * COM port.
	 * 
	 * @param comPort the com port to use (e.g. COM2)
	 * 
	 * @return true, if serial connection is started successfully, false othervise.
	 * 
	 * @see org.ubicollab.service.X10Proxy.X10Proxy#startSerialConnection(java.lang.String)
	 */
	public boolean startSerialConnection(String comPort) {
		if (! connected ) {
			try {
				x10Controller = new X10Impl(comPort);
				connected = true;
			}
			catch (Exception e) {
				connected = false;
				e.printStackTrace();
			}
		}
		return connected;
	}

	

	/**
	 * Stop the ongiong serial connection.
	 * 
	 * @see org.ubicollab.service.X10Proxy.X10Proxy#stopSerialConnection()
	 */
	public void stopSerialConnection() {
		if ( connected ) {
			try {
				x10Controller.close();
				x10Controller = null;
				connected = false;
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}



	/**
	 * Send a brighten or dim command to the X10 device identified by 
	 * <code>house</code> and <code>unit</code>.
	 * 
	 * @see org.ubicollab.service.X10Proxy.X10Proxy#dimDevice(java.lang.String, int, int, int)
	 */
	public void dimDevice(String house, int unit, int cmnd, int percentage) {
		if ( connected ) {
			x10Controller.dimDevice(house.charAt(0), unit, cmnd, percentage);
		}
	}

	
}