package org.ubicollab.service.X10Proxy;

/**
 * A X10 proxy service created for test and demo purpose in UbiCollab. This 
 * bundle has a prerequisite: It requires that the Java comm API is installed 
 * on the computer it runs on.
 * 
 * @author Kim-Steve Johansen
 * @version 0.1
 */
public interface X10Proxy {

	/**
	 * Start a serial connection to a CM11A serial X10 controller, on the
	 * provided COM port.
	 * 
	 * @param comPort the com port to use, e.g. COM1
	 * 
	 * @return a string describing the connection result.
	 */
	public boolean startSerialConnection(String comPort);

	
	/**
	 * Stop the ongoing serial connection and relase resources used.
	 */
	public void stopSerialConnection();
	
	
	/**
	 * Method to send an X10 on/off command to a device with a given
	 * address.
	 * <p>
	 * This method assumes that a serial connection to a X10 device already 
	 * have been started. Nothing will hapen if one does not exist.
	 * 
	 * @param turnOn the turn on
     * @param unit the unit address (1-16)
     * @param house the house address (A-P)
	 * 
	 */
	public void sendCommand(String house, int unit, boolean turnOn);
	
	
    /**
     * Method to send dim or brighten commands to the given
     * X10 device.
	 * <p>
	 * This method assumes that a serial connection to a X10 device already 
	 * have been started. Nothing will hapen if one does not exist.
     * 
     * @param cmnd Either 0 for dim or 1 for brighten
     * @param unit the unit address (1-16)
     * @param house the house address (A-P)
     * @param percentage the level to change between 1 and 100 (persentage).
     */
    public void dimDevice(String house, int unit, int cmnd, int percentage);
	
	
}
